# QA Launchpad Bug Reports
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Models for qlbr."""

import re

from django.db import models


class SearchTag(models.Model):

    """SearchTag for ManyToMany in Searches."""

    search_tag = models.CharField(max_length=255, primary_key=True)

    def __unicode__(self):
        return self.search_tag


class Item(models.Model):

    """Item for ManyToMany in criteria."""

    item = models.CharField(max_length=255, primary_key=True)

    def __unicode__(self):
        return self.item


class Criterion(models.Model):

    """Crtieria for searches."""

    key = models.CharField(max_length=255)
    is_list = models.BooleanField()
    value = models.CharField(max_length=255)
    values = models.ManyToManyField('Item')

    def __unicode__(self):
        return self.key

    @property
    def get(self):
        """Provide value or list.

        :returns: value of this criterion
        :rtype: str or list

        """
        if self.is_list:
            return list(self.values.values_list('item', flat=True))
        else:
            return self.value


class Project(models.Model):

    """Project for searches."""

    project = models.CharField(max_length=255, primary_key=True)

    def __unicode__(self):
        return self.project


class Search(models.Model):

    """Description of a search for launchpad bugs."""

    name = models.CharField(max_length=255)
    search_tags = models.ManyToManyField('SearchTag')
    projects = models.ManyToManyField('Project')
    criteria = models.ManyToManyField('Criterion')
    last_update = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return self.name

    @property
    def criteria_dict(self):
        """Provide dictionary of criteria to pass to launchpad API.

        :returns: dictionary of criteria
        :rtype: dict

        """
        criteria = {}
        for criterion in self.criteria.iterator():
            criteria[criterion.key] = criterion.get
        return criteria


class Person(models.Model):

    """Person."""

    self_link = models.CharField(max_length=255, primary_key=True)
    web_link = models.CharField(max_length=255)
    name = models.CharField(max_length=255)
    display_name = models.CharField(max_length=255)
    last_update = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return self.display_name


class Tag(models.Model):

    """Tag for bugs."""

    tag = models.CharField(max_length=255, primary_key=True)
    matcher = re.compile('rls-[a-z]-incoming')

    def __unicode__(self):
        return self.tag

    @property
    def release(self):
        """Indicate whether this is a release tag.

        :returns: whether this is a release tag
        :rtype: bool

        """
        if self.matcher.search(self.tag):
            return True
        else:
            return False


class Bug(models.Model):

    """Bug."""

    self_link = models.CharField(max_length=255, primary_key=True)
    web_link = models.CharField(max_length=255)
    bug_id = models.IntegerField()
    tags = models.ManyToManyField('Tag')
    date_last_updated = models.DateTimeField()
    date_created = models.DateTimeField()
    owner = models.ForeignKey('Person', on_delete=models.PROTECT)
    duplicate_of = models.ForeignKey(
        'self', blank=True, null=True, on_delete=models.PROTECT)
    # Some apport-reported bugs have titles > 500 so use TextField.
    title = models.TextField()
    last_update = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return str(self.bug_id)

    @property
    def tag_list(self):
        """Provide tags as a list.

        :returns: list of tags
        :rtype: list

        """
        return ' '.join(self.tags.values_list('tag', flat=True))

    @property
    def triaged(self):
        """Indicate whether this bug is triaged.

        :returns: whether this bug is triaged
        :rtype: bool

        """
        if self.task_set.filter(status='Fix Released').exists():
            return True
        if any([tag.release for tag in self.tags.all()]):
            return True
        return False


class Task(models.Model):

    """Bug task."""

    self_link = models.CharField(max_length=255, primary_key=True)
    web_link = models.CharField(max_length=255)
    importance = models.CharField(max_length=255)
    status = models.CharField(max_length=255)
    date_created = models.DateTimeField()
    bug_target_name = models.CharField(max_length=255)
    bug_target_display_name = models.CharField(max_length=255)
    bug = models.ForeignKey('Bug', on_delete=models.PROTECT)
    owner = models.ForeignKey('Person', on_delete=models.PROTECT,
                              related_name='task_owner')
    assignee = models.ForeignKey(
        'Person', blank=True, null=True, on_delete=models.PROTECT,
        related_name='task_assignee')
    search_tags = models.ManyToManyField('SearchTag')
    last_update = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return self.web_link

    @property
    def triaged(self):
        """Indicate whether this task is triaged.

        :returns: whether this task is triaged
        :rtype: bool

        """
        if self.bug.triaged:
            return True
        if self.status == 'New':
            return False
        return True
