# QA Launchpad Bug Reports
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Test the get_person command."""

from mock import patch, MagicMock
import unittest

from django.core.exceptions import ObjectDoesNotExist
from launchpadlib.launchpad import Launchpad

from qlbr.management.commands.get_task_data import Command


class CommandTestCase(unittest.TestCase):

    """Mock a Command enclosing code under test.

    Refer to a member function as self.command.do_something, e.g.

    """

    def setUp(self):
        command_mock = MagicMock(
            spec=Command,
            return_value=None)
        self.command_patch = patch.object(
            Command,
            '__init__',
            new=command_mock)
        self.command_patch.start()
        self.command = Command()
        self.command.lp = MagicMock(
            spec=Launchpad)

    def tearDown(self):
        self.command_patch.stop()


class TestGetPerson(CommandTestCase):

    """Test the get_person command."""

    def test_none(self):
        self.assertIsNone(self.command.get_person(None),
                          'Calling get_person with None did not return None')

    def test_fetched_person(self):
        with patch('qlbr.management.commands.get_task_data.Person') \
                as Person, patch.object(self.command.lp, 'load') as lp_load:
            query = 'Test'
            retval = 'Test Person'
            Person.objects.get.return_value = retval
            Person.side_effect = Exception(
                'Person created for locally stored person')
            lp_load.side_effect = Exception(
                'Launchpad called for locally stored person')
            self.assertEqual(
                self.command.get_person(query), retval,
                'get_person did not return as expected for stored person')

    def test_fetch_person(self):
        with patch('qlbr.management.commands.get_task_data.Person') \
                as Person, patch.object(self.command.lp, 'load') as lp_load:
            lp_load.return_value = MagicMock(self_link='fake_self_link',
                                             web_link='fake_web_link',
                                             display_name='fake_display_name')
            # Unfortunately we can't pass 'name' as an init param :( .
            lp_load.return_value.name = 'fake_name'
            query = 'Test'
            Person.objects.get.side_effect = ObjectDoesNotExist()
            self.command.get_person(query)
            lp_load.assert_called_once_with(query)
            Person.assert_called_once_with(
                self_link='fake_self_link',
                web_link='fake_web_link',
                name='fake_name',
                display_name='fake_display_name')
