# QA Launchpad Bug Reports
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Provide bug report views."""

import datetime
import json
import re

from django.http import HttpResponse
from django.shortcuts import render

from qlbr import TZ
from qlbr.models import Task
from qlbr.team_mapping import (
    TEAM_MAPPING,
    PROJECT_MAPPING,
)

OPEN_STATUSES = [
    'New',
    'Confirmed',
    'Triaged',
    'In Progress',
    'Fix Committed',
    'Incomplete',
]


def index(request, task_filter=None, triaged=None):
    """Main view of all bugs.

    :returns: view of all bugs
    :rtype" HttpResponse

    """
    team_stats = {'Other': ['Other', 'Other', 0, 0, 0]}
    for team in TEAM_MAPPING:
        team_stats[team] = [
            team,
            team.replace(' ', '').replace('/', ''),
            0,
            0,
            0,
        ]
    tasks = {}
    if task_filter is None:
        task_objects = Task.objects
    else:
        task_objects = Task.objects.filter(**task_filter)
    task_objects = task_objects.filter(bug__tags__tag='rtm14')
    if 'tag' in request.REQUEST:
        for tag in request.REQUEST['tag'].split():
            task_objects = task_objects.filter(bug__tags__tag=tag)
    task_iterator = task_objects.iterator()
    for task in task_iterator:
        bugid = task.bug.bug_id
        report_task = {
            'title': task.bug.title,
            'taglist': task.bug.tag_list,
            'date_created': task.date_created.isoformat(),
            'age': (datetime.datetime.now(TZ) - task.date_created).days,
            'importance': task.importance,
            'status': task.status,
            'web_link': task.web_link,
            'duplicate_link': '',
            'owner': task.owner.display_name,
            'triaged': task.triaged,
        }
        if triaged is not None:
            if task.triaged != triaged:
                continue
        dupe_of = task.bug.duplicate_of
        if dupe_of:
            report_task['duplicate_link'] = ('<a href="{}">{}</a>'
                                             .format(task.web_link, bugid))
            report_task['owner'] += ' (dup)'
            bugid = dupe_of.bug_id
            report_task['web_link'] = dupe_of.web_link
            report_task['triaged'] = dupe_of.triaged
            report_task['status'] = dupe_of.task_set.first().status
            report_task['target'] = (dupe_of.task_set.first()
                                     .bug_target_display_name)

        # Pilfered from collect-bug-data in lp:arsenal
        m = re.search('(\S+)\s\(.*\)', task.bug_target_display_name)
        if m is not None:
            report_task['target'] = m.group(1)
            report_task['target_display_name'] = m.group(1)
        else:
            report_task['target'] = task.bug_target_display_name
            report_task['target_display_name'] = task.bug_target_display_name

        if task.assignee:
            report_task['assignee'] = task.assignee.display_name
        else:
            report_task['assignee'] = 'Unassigned'
        if bugid not in tasks:
            tasks[bugid] = []

        target = report_task['target'].replace(' (Ubuntu)', '')
        target = target.replace(' ', '').replace('-', '')
        target = target.lower()
        if target in PROJECT_MAPPING:
            team = PROJECT_MAPPING[target]
        else:
            team = 'Other'
        if report_task['status'] in OPEN_STATUSES:
            team_stats[team][2] += 1
            if report_task['importance'] == 'Critical':
                team_stats[team][3] += 1
            if report_task['importance'] == 'High':
                team_stats[team][4] += 1
        report_task['target'] = team.replace(' ', '').replace('/', '')
        tasks[bugid].append(report_task)
        targets = {task['target'] for task in tasks[bugid]}
        if 'Other' in targets and targets != {'Other'}:
            for task in filter(
                    lambda t: t['target'] == 'Other'
                    and t['status'] in OPEN_STATUSES,
                    tasks[bugid]):
                team_stats['Other'][2] -= 1
                if task['importance'] == 'Critical':
                    team_stats['Other'][3] -= 1
                if task['importance'] == 'High':
                    team_stats['Other'][4] -= 1
            tasks[bugid] = list(
                filter(lambda t: t['target'] != 'Other', tasks[bugid]))

    team_report_order = []
    for team in sorted(list(team_stats)):
        team_report_order.append(team_stats[team])
    tasks_json_dump = json.dumps(tasks, sort_keys=True, indent=4)
    context = {'report_title': 'RTM Bugs',
               'tasks': tasks,
               'timestamp': datetime.datetime.now().isoformat(),
               'datefilter': True,
               'target_report_order': team_report_order,
               'tasks_json_dump': tasks_json_dump}
    if 'json' in request.REQUEST:
        return HttpResponse(tasks_json_dump, mimetype='application/json')
    else:
        return render(request, 'qlbr/qa-default.html', context)
